import 'dart:io';
import 'dart:isolate';
import 'package:async/async.dart';
import 'package:yona/yona.dart';
import 'dart:math' as math;
import 'package:dart_console/dart_console.dart' as console;
Future<ReceivePort> bench(NetAddr host, int n, int limit) async {
  var port = ReceivePort();
  Isolate.spawn((port) async {
    List timings = [];
    for (var i = 0; i < n; i++) {
      var user1 = await Crypto.genSK();
      var user2 = await Crypto.genSK();
      var packet = Packet(Crypto.derivePK(user1), Crypto.derivePK(user2),
          await Crypto.randBytes(100), Packet.zeroID);
      var zeroPacket = Packet(
          Crypto.derivePK(user1), Crypto.derivePK(user1), [], Packet.zeroID);
      List<Packet> packets = [zeroPacket];
      var startSend = DateTime.now();
      await Longlog.push(host, packet.encode(), user2, Duration(seconds: 2),Duration(minutes: 10));
      var endSend = DateTime.now();
      var startRecv = DateTime.now();
      packets = await Longlog.pull(
          host,
          zeroPacket.encode().sublist(0, 1 + Packet.pkSize + 8),
          limit,
          Duration(seconds: 2));
      var endRecv = DateTime.now();
      assert(packets.length == math.min(i + 1, limit));
      for (var pack in packets) {
        assert(Utils.equal(pack.data, packet.data));
        assert(Utils.equal(pack.from, packet.from));
        assert(Utils.equal(pack.to, packet.to));
      }
      timings.add({
        "i": i,
        "recv": endRecv.difference(startRecv).inMicroseconds,
        "send": endSend.difference(startSend).inMicroseconds
      });
      (port as SendPort).send(timings);
    }
    Isolate.exit();
  }, port.sendPort,errorsAreFatal: true);
  return port;
}
var timings = [];
void result(console.Console terminal,dynamic timing){
  timings.addAll(timing);
  for (var prop in ["recv","send"]) {
    var min = (timings
        .map((e) => e[prop])
        .reduce((a, b) => math.min(a as int, b as int)) /
        Duration.microsecondsPerSecond as double)
        .toStringAsPrecision(4);
    var max = (timings
        .map((e) => e[prop])
        .reduce((a, b) => math.max(a as int, b as int)) /
        Duration.microsecondsPerSecond as double)
        .toStringAsPrecision(4);
    var total = (timings.map((e) => e[prop]).reduce((a, b) => a + b) ~/
        Duration.microsecondsPerSecond);
    var avg = (timings.map((e) => e[prop]).reduce((a, b) => a + b) /
        timings.length /
        Duration.microsecondsPerSecond as double)
        .toStringAsPrecision(4);
    terminal.writeLine("$prop avg: $avg min: $min max: $max total: $total");
  }
}
void main(List<String> arguments) async {
  var host = NetAddr.fromString(arguments[arguments.length - 1]);
  var n = int.parse(arguments[arguments.indexOf("-n")+1]);
  var c = int.parse(arguments[arguments.indexOf("-c")+1]);
  var l = int.parse(arguments[arguments.indexOf("-l")+1]);
  var port = StreamGroup.merge(Iterable.generate(c,(_) async*{
      var port =  await bench(host, n, l);
      yield* port.where((event) {
        if(event == null){
          port.close();
          return false;
        }
        return true;
      });
  }));
  port.handleError((e)=>print(""));
  var terminal = console.Console();
  terminal.clearScreen();
  terminal.resetCursorPosition();
  var i = 0;
  await port.forEach((timings) {
    terminal.resetCursorPosition();
    terminal.clearScreen();
    terminal.writeLine(host.toString());
    terminal.writeLine("c: $c n: $n l: $l");
    var total = n*c;
    result(terminal,timings);
    if (++i >= total){
      exit(0);
    }
    terminal.writeLine("${(i/total*100).toStringAsPrecision(2)}% $i/$total");
    terminal.writeLine();
    terminal.writeLine();
    terminal.writeLine();
    terminal.writeLine();
    Future.delayed(Duration(milliseconds: 100));
  });
}
